include winim/[inc\winuser]
import functions        # eigene Procedures ausgelagert
import BinToBase64, res # bin64-Ressourcen & Umwandlungsfunktion
import os, parseopt, parsecfg, tables, strutils
import wNim/[wApp, wFrame, wPanel, wTextCtrl, wComboBox, wStaticText, 
             wCheckBox, wButton, wToolTip, wFont, wImage, wIcon, wBitmap,
             wStaticBitmap]

#========== DEBUG - MODE ============================================
var debug = false                  # für "echo DEBUG" auf true setzen
#====================================================================

#========== cmd-Line auswerten ======================================
# erwartet wird kein oder ein einzelner Parameter (Suchstring)
let cmdLine = commandLineParams()
if debug: echo "cmdLIne: ", cmdLine
var 
  search_item: string
  p = initOptParser(cmdLine)
for kind, key, val in p.getopt(): 
  if kind == cmdArgument: search_item = key
if debug: echo "cmdLine-SearchItem: ", search_item
#====================================================================

#========== ini - initialization ====================================
let ini_file = CurrentDir().joinPath("websearch.ini")
var ini: Config

if not ini_file.fileExists():
  echo "None INI"
  ini = newConfig()
  ini.setSectionKey("engines","Google",r"https://www.google.de/search?q=")
  ini.setSectionKey("engines","Startpage",r"https://www.startpage.com/do/dsearch?query=")
  ini.setSectionKey("sites","AutoIt-DE","autoit.de")
  ini.setSectionKey("sites","AutoIt-EN","autoitscript.com")
  ini.setSectionKey("sites","MSDN",r"msdn.microsoft.com")
  ini.writeConfig(ini_file)

ini = loadConfig(ini_file)
#====================================================================

#========== websearch window ========================================
type
  CtrlID = enum  # Festlegen der ID für Controls 
    idEngines = wIdUser, idSites, idItem, idNewVal

var 
  combo_change: wComboBox       # Combo, in der aktuell neuer Eintrag erfolgte
  ini_has_changed: bool = false # wenn neuer Eintrag -- bei Beenden speichern

let
  wsApp = App()
  ico_frame = Icon(ico_web_bin.Base64ToBin())
  ico_help = Icon(ico_help_bin.Base64ToBin())
  wsFrame = Frame(title="WEB - SUCHE",size=(600,280))
  wsPanel = wsFrame.Panel()
  wsBtnExit = wsPanel.BitmapButton(ico_exit_bin.Base64ToBin(), size=(24,24), 
                                   style=wBorderRaised)
  wsLabelEngine = wsPanel.StaticText(label="Suchmaschine")
  wsCheckSite = wsPanel.CheckBox(label=" Suchen auf Seite [optional]")
  wsLabelItem = wsPanel.StaticText(label="Suchbegriff")
  wsBtnNewVal = wsPanel.Button(label="Speichere neuen Eintrag")
  wsBtnHelp = wsPanel.Button(label="Hilfe")
  wsBtnSearch = wsPanel.Button(label="Suchen")
  wsComboEngines = wsPanel.ComboBox(id=idEngines,style=wCbDropDown)
  wsComboSites = wsPanel.ComboBox(id=idSites,style=wCbDropDown)
  wsItem = wsPanel.TextCtrl(value=search_item,id=idItem,style=wBorderSunken)
  wsNewValue = wsPanel.TextCtrl(id=idNewVal,style=wBorderSunken)
  wsHelp = wsFrame.Frame(title="Hilfe",size=(500,300), 
                         style=wCaption or wPopupWindow)
  wsHelpPanel = wsHelp.Panel()
  text_help = # ACHTUNG! Zeilenumbruch in Windows-Editfeld: \r\n
    "Erfassen neuer Suchmaschinen / Suchseiten\r\n\r\n" &
    "In der entsprechenden Combobox einen neuen Displaynamen für die Suchmaschine oder -seite eintragen und mit <ENTER> bestätigen.\r\n" &
    "Eine Textfeld wird eingeblendet, in dem die suchmaschinenspezifische " & "Adresszeile (z.B. \"www.yandex.com/search/?text=\") oder die Adresse " & "der Webseite eingetragen werden.\r\n" &
    "Abschließend den neuen Eintrag speichern."
  font_courier = Font(faceName="Courier New", pointSize=12)
  wsHelpText = wsHelp.TextCtrl(value=text_help,
                      style=wBorderSunken or wTeMultiLine or wTeReadOnly)

wsHelpText.setFont(font_courier) # Font für Hilfetext
wsFrame.disableCloseButton()     # Close-Button Suchfenster deaktivieren
wsFrame.setIcon(ico_frame)
wsHelp.setIcon(ico_help)
wsNewValue.TextCtrlSetCueBanner(
           "Wert für den neuen Combobox-Eintrag hier erfassen")
wsNewValue.hide()
wsBtnNewVal.hide()

proc ComboFill(combo: wComboBox, sec: OrderedTableRef[string,string]) =
  for k, _ in sec.pairs(): combo.append(k) 
  combo.select(0)

wsComboEngines.ComboFill(ini.Section("engines"))
wsComboSites.ComboFill(ini.Section("sites"))  

proc AppExit() =
  if debug: echo "exit - ini_has_changed: ", ini_has_changed
  if ini_has_changed: 
    if debug: echo "Write to INI"
    ini.writeConfig(ini_file)
  wsFrame.delete()

proc KeyByComboSelection(combo: wComboBox, sec: string): string =
  var index = combo.getSelection()
  if index < 0: index = 0 
  result = ini.Section(sec).Key(combo.getText(index))

proc Search() =
  if not wsItem.isEmpty():
    let 
      text = wsItem.getValue()
      engine = wsComboEngines.KeyByComboSelection("engines")
    var site = "%20site:" & wsComboSites.KeyByComboSelection("sites")
    if not wsCheckSite.isChecked(): site = ""
    let cmd = engine & text & site
    if debug: echo "Search: [", text, "] cmd: ", cmd
    MyShellExecute(file=cmd)
    AppExit()
  else: wsItem.setValue("SUCHBEGRIFF FEHLT!")

wsFrame.wEvent_TextEnter do (event: wEvent):  # ENTER in einem Textfeld
  let sender = event.getEventObject()         # wer hat das Event gesendet
  case sender.getId()                         # welche ID hat das Ctrl
  of idEngines: 
    if debug: echo "wEvent_TextEnter - Engines"
    combo_change = wsComboEngines
  of idSites: 
    if debug: echo "wEvent_TextEnter - Sites"
    combo_change = wsComboSites
  of idItem: 
    if debug: echo "wEvent_TextEnter - Search-Item"
    Search()
  else: discard
  wsNewValue.show()
  wsBtnNewVal.show()

wsBtnSearch.wEvent_Button do ():
  Search()

wsBtnHelp.wEvent_Button do ():
  wsHelp.show()

wsBtnNewVal.wEvent_Button do ():
  # Wurde ein Wert erfasst
  let new_val = wsNewValue.getValue()
  if new_val == "": wsFrame.MessageBox("Für den neuen Eintrag wurde kein Wert erfasst!" ,"FEHLER", MB_OK or MB_ICONERROR)
  else:
    # Ini Sektion ermitteln, in der der neue Wert eingetragen wird
    var section = "engines"
    if combo_change.id == idSites: section = "sites"
    # Wert aus Combo, in der geändert wurde, auslesen (wEvent_TextEnter)
    let combo_val = combo_change.getValue() 
    # Prüfen ob der Wert noch nicht existiert (Index von Itemliste abfragen)
    let idx = combo_change.findText(combo_val)
    if debug: echo "gesuchter Eintrag Combo[", section, "]: ", 
                                     combo_val, " - Index: ", idx
    if idx == -1:  # Wert existiert noch nicht
      # der Combo-Liste hinzufügen
      combo_change.append(combo_val)
      # in Ini-Table Displayname und Wert eintragen
      ini.setSectionKey(section, combo_val, new_val)
      # Speichern der Ini bei Beenden veranlassen
      ini_has_changed = true
      if debug: echo "ini_has_changed: ", ini_has_changed
    else: wsFrame.MessageBox("Der Eintrag in der ComboBox \"" & 
                  section.capitalizeAscii() & "\" existiert bereits!",
                  "Achtung!", MB_OK or MB_ICONSTOP)
  wsNewValue.clear()
  wsNewValue.hide()
  wsBtnNewVal.hide()

proc helpLayout() =
  wsHelp.layout:
    wsHelpPanel:
      width = wsHelp.width
      height = wsHelp.height
    wsHelpText:
      left = 15
      top = 15
      width = wsHelpPanel.width - 30
      height = wsHelpPanel.height - 30

proc layout() =
  wsFrame.layout:
    wsPanel:
      width = wsFrame.width
      height = wsFrame.height
    wsBtnExit:
      right = wsFrame.width - 15
      top = 5
    wsLabelEngine:
      left = 15
      top = 15
    wsCheckSite:
      left = wsFrame.width / 2 + 15
      top = 15
    wsComboEngines:
      left = 15
      top = wsLabelEngine.bottom + 10
      width = wsFrame.width / 2 - 30
    wsComboSites:
      left = wsCheckSite.left
      top = wsComboEngines.top
      width = wsComboEngines.width
    wsLabelItem:
      left = 15
      top = wsComboEngines.bottom + 15
    wsItem:
      left = 15
      top = wsLabelItem.bottom + 10
      width = wsFrame.width - 30
    wsNewValue:
      left = 15
      top = wsItem.bottom + 10
      width = wsFrame.width - 30
    wsBtnNewVal:
      left = 15
      top = wsNewValue.bottom + 15
      width = wsBtnNewVal.getBestSize().width
    wsBtnHelp:
      centerX = wsFrame.centerX
      top = wsBtnNewVal.top
      width = wsBtnNewVal.width
    wsBtnSearch:
      right = wsFrame.width - 15
      top = wsBtnNewVal.top
      width = wsBtnNewVal.width

wsBtnSearch.setFocus()

wsFrame.wEvent_Size do ():
  layout()

wsBtnExit.wEvent_CommandLeftClick do ():
  AppExit()

wsFrame.wIdExit do ():  discard

wsHelp.wIdExit do ():
  wsHelp.hide()

wsHelp.minSize = (500,300)
wsHelp.maxSize = (500,300)
wsHelp.center()
helpLayout()

wsFrame.minSize = (600,280)
wsFrame.maxSize = (int(GetSystemMetrics(SM_CXSCREEN)),280)
layout()
wsFrame.center()
wsFrame.show()

wsApp.mainLoop()
#====================================================================

